%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Author: Patrick Kano
%
%
%Date: November 1, 2005
%
%
%Description: Weeks Method for e^{A}b with Shur Factorization
%
%
%Reference: 
%Kano, P.,M. Brio, and J. Moloney.  ``Application of Weeks method for the numerical inversion of
%the Laplace transform to the matrix exponential", Communications in mathematical sciences", vol. 3, no. 3,
%pp. 335-372, 2005.
%
%
%Input:
%Amatrix = Input Matrix
%
%Bvector = Input Vector/Initial Condition
%
%NLaguerreCoef = Number of Laguerre Coefficients
%
%Parameter Switch 
%1) MinMax Method: Grid Search 
%2) SVD Method: rmin Estimate
%3) Weideman Method
%
%SigmaDistance = Search (Abscissa,Abscissa+SigmaDistance) for optimal parameter value
%
%DeltaSimga = Sigma Step Size
%
%BetaMin = Search (BetaMin,BetaMax) for optimal parameter value
%
%BetaMax = Search (BetaMin,BetaMax) for optimal parameter value
%
%DeltaBeta = Change in Beta
%
%ErrorSwitch = Estimate Errors 1-On 0-Off
%
%Laguerre Coefficient Computation Method Switch = 0-Midpoint Rule, 1-FFT
%
%
%Output: 
%Solnvector = Solution Vector e^{A}b
%
%SigmaWeeks = Algorithm selected optimal parameter \sigma
%
%BetaWeeks = Algorithm selected optimal parameter b
%
%EstimatedTotalError = Estimated Total Absolute Error
%
%TruncationError = Estimated Truncation Absolute Error
%
%RoundoffError = Estimated Roundoff Absolute Error
%
%
%Comments: 
%
%1)The Weeks method parameters (\sigma,b) are given the names (\sigma,\beta).
%
%2)Only the 3 most reliable parameter selection algorithms are included.  See the reference for additional
%information on the less reliable algorithms.  
%
%3)In general, the code has been optimized and unnecessary variables and computations eliminated.
%
%4)In the original referenced work, an arbitrary choice of the (\sigma,b) plain was studied.  This
%code allows the user to specify a range of parameter values (\betamin,\betamax).  The minimum
%\sigma value is abscissa of convergence = largest eigenvalue of the matrix A, the largest is
%(abscissa+\sigmadistance).
%
%5)To incorporate a time step, multiply the matrix A by the time step before submitting to the code.
%
%6) Warning messages have been turned off.  This command can be commented out.
%
%Input Format:
%
%[Solnvector,SigmaWeeks,BetaWeeks,EstimatedTotalError,TruncationError,RoundoffError] =...
%WeeksSolverMainPub(Amatrix,Bvector,NLaguerreCoef,ParameterSwitch,...
%SigmaDistance,DeltaSigma,BetaMin,BetaMax,DeltaBeta,ErrorSwitch,LagSwitch)
 
function [Solnvector,SigmaWeeks,BetaWeeks,EstimatedTotalError,TruncationError,RoundoffError] =...
WeeksSolverMainPub(Amatrix,Bvector,NLaguerreCoef,ParameterSwitch,...
SigmaDistance,DeltaSigma,BetaMin,BetaMax,DeltaBeta,ErrorSwitch,LagSwitch)

format long;
format compact;

warning('off');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%DO NOT EDIT BELOW THIS LINE%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
MInversions = NLaguerreCoef; %Number of Contour Points, Do NOT modify.

NNodes = size(Bvector,1); 
if(NNodes~=size(Amatrix,2)) error('Check sizes of matrix and vector!'); end

%Schur Decomposition and Parameters
if(ParameterSwitch==1)
 %disp('MinMax Method'); %MinMax Method: Grid Search %Note: Same algorithm as in the matrix exponential 
 [SigmaWeeks,BetaWeeks,Unitmatrix,Trianmatrix] = ...
 WeeksMinMaxGridParam(Amatrix,SigmaDistance,DeltaSigma,BetaMin,BetaMax,DeltaBeta);
elseif(ParameterSwitch==2)
 %disp('Pseudospectrum Method'); %SVD Method: rmin Estimate %Note: Same algorithm as in the matrix exponential
 [SigmaWeeks,BetaWeeks,Unitmatrix,Trianmatrix] =...
 WeeksPseudorminParam(Amatrix,NLaguerreCoef,SigmaDistance,DeltaSigma,BetaMin,BetaMax,DeltaBeta);
elseif(ParameterSwitch==3)
 %disp('Weideman Method'); %Note: Different algorithm as in the matrix exponential
 [SigmaWeeks,BetaWeeks,Unitmatrix,Trianmatrix,EstimatedTotalError] =...
 WeeksWeidemanParam(Amatrix,Bvector,NLaguerreCoef,MInversions,LagSwitch,SigmaDistance,DeltaSigma,BetaMin,BetaMax,DeltaBeta);
else
 error('Problem with the Choice of Parameter Selection Method');
end

%Change Basis
Bvector = Unitmatrix\Bvector;

%Compute the Laguerre Coefficients
if(LagSwitch==0)
%disp('Midpoint Rule for Laguerre Coefficients');
[LaguerreCoef] = WeeksMidPntLagCoef(Unitmatrix,Trianmatrix,Bvector,NNodes,NLaguerreCoef,MInversions,SigmaWeeks,BetaWeeks);
elseif(LagSwitch==1)
%disp('FFT for Laguerre Coefficients');
[FFTSamples] = WeeksFFTSample(Unitmatrix,Trianmatrix,Bvector,NNodes,NLaguerreCoef,SigmaWeeks,BetaWeeks);
[LaguerreCoef] = WeeksFFTLagCoef(FFTSamples,NNodes,NLaguerreCoef);
clear FFTSamples;
else
error('Error in choice of LaguerreCoefficients Methods');
end

%Estimate Error
if(ErrorSwitch==1)
[TruncationError,RoundoffError,EstimatedTotalError] =...
WeeksErrorEstimate(Unitmatrix,Trianmatrix,Bvector,NNodes,NLaguerreCoef,MInversions,SigmaWeeks,BetaWeeks,LagSwitch)
else
TruncationError = 0; RoundoffError=0; EstimatedTotalError = 0;
end

%Clenshaw Algorithm Summation
[Solnvector] = WeeksClenshawAlgorithm(NNodes,NLaguerreCoef,SigmaWeeks,BetaWeeks,LaguerreCoef);

%Return to Original Basis
Solnvector(:,1) = Unitmatrix*Solnvector(:,1);




